package cn.jisol.game.network;

import cn.jisol.game.actions.GActionEnum;
import cn.jisol.game.data.Cache;
import cn.jisol.game.network.client.GClient;
import cn.jisol.ngame.actions.core.NActionEnum;
import cn.jisol.ngame.client.NClient;
import cn.jisol.ngame.network.JNetwork;
import org.springframework.stereotype.Controller;

import javax.websocket.OnClose;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.ServerEndpoint;
import java.io.InputStream;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;

@ServerEndpoint(
    value = "/websocket/{token}"
)
@Controller
public class WebSocket {

    public static final Map<String, NClient> CLIENTS = new ConcurrentHashMap<>();

    @OnOpen
    public void onOpen(Session session){

        String token = session.getPathParameters().get("token");
        NClient client = new GClient(session);
        if(Objects.isNull(Cache.TOKEN.get(token))){
            //发送Token过期请求
            client.invoke(GActionEnum.TOKEN_EXPIRED);
            //关闭连接
            client.Close();
            return;
        }
        CLIENTS.put(session.getId(),client);
        System.out.printf("[WebSocket] %s 连接成功.\n",session.getId());

    }

    @OnMessage
    public void onMessage(Session session, InputStream inputStream){
        JNetwork.onMessage(inputStream,CLIENTS.get(session.getId()),CLIENTS);
    }

    @OnClose
    public void onClose(Session session){
        CLIENTS.remove(session.getId());
    }

}
